package de.fraunhofer.iem.dataprovider.taskManager.tasks

import de.fraunhofer.iem.dataprovider.logger.getLogger
import de.fraunhofer.iem.dataprovider.taskManager.events.Event
import de.fraunhofer.iem.dataprovider.taskManager.events.GroupTaskFailedEvent
import de.fraunhofer.iem.dataprovider.taskManager.events.TaskFailedEvent
import java.util.*

interface ITask {
    suspend fun run()
    val taskID: UUID
}

abstract class Task : ITask {
    override val taskID: UUID = UUID.randomUUID()
    protected open val groupID: UUID? = null

    protected val logger = getLogger(javaClass)
    protected abstract val responseChannel: suspend (event: Event) -> Unit
    protected abstract suspend fun execute()

    override suspend fun run() {
        try {
            execute()
        } catch (e: Throwable) {
            logger.error("Task execute failed.")
            sendTaskFailedEvent(e)
            throw e
        }
    }

    protected suspend fun sendTaskFailedEvent(e: Throwable) {
        if (groupID != null) {
            responseChannel(GroupTaskFailedEvent(e, groupID!!, taskID))
        } else {
            responseChannel(TaskFailedEvent(e))
        }
    }
}
