package de.fraunhofer.iem.dataprovider.taskManager.tasks.tools.occmd

import de.fraunhofer.iem.dataprovider.repository.service.RepositoryService
import de.fraunhofer.iem.dataprovider.taskManager.events.Event
import de.fraunhofer.iem.dataprovider.taskManager.tasks.tools.ToolProcessTask
import de.fraunhofer.iem.dataprovider.taskManager.tasks.tools.occmd.enumeration.Checks
import de.fraunhofer.iem.dataprovider.taskManager.tasks.tools.occmd.json.raw.RawResultJson
import kotlinx.serialization.json.Json
import java.nio.file.Path
import java.util.*

class OccmdTask(
    repositoryDirectoryPath: String,
    override val repoId: UUID,
    override val responseChannel: suspend (event: Event) -> Unit,
    private val repositoryService: RepositoryService,
    override val outputDirectory: String
) : ToolProcessTask<List<RawResultJson>>() {

    override val flags: Array<String> = arrayOf(repositoryDirectoryPath, getRepositoryId(repoId))

    override val resultFileOutputPath: Path =
        Path.of(outputDirectory, "occmd-${System.currentTimeMillis()}-${this.taskID}.json")

    override suspend fun parseProcessResults(resultPath: Path): List<RawResultJson> {
        val rawResultJson = mutableListOf<RawResultJson>()
        val json = Json { ignoreUnknownKeys = true }
        val resFile = resultPath.toFile()
        if (resFile.exists()) {
            resFile.forEachLine {
                rawResultJson.add(json.decodeFromString<RawResultJson>(it))
            }
        }
        return rawResultJson
    }

    override suspend fun storeResultsInDb(result: List<RawResultJson>) {
        result.forEach { r ->
            val check = Checks.fromString(r.check)
//            when (check) {
////                Checks.CheckedInBinaries ->
//            }
        }
    }

    private fun getRepositoryId(repoId: UUID): String {
        val repoPlatformId = repositoryService.findRepoByID(repoId)?.repoId
            ?: throw NoSuchElementException("Repository with id $repoId was not found.")
        return repoPlatformId.toString()
    }

}

//class CloneGitTask(
//    private val gitRepository: RepositoryCreateDto,
//    private val repoDbId: UUID,
//    override val responseChannel: suspend (task: Event) -> Unit,
//    private val outputPath: String,
//) : Task() {
//
//    override suspend fun execute() {
//
//        val outputDirName = "${gitRepository.name}-${taskID}"
//        val outputDirectory = Paths.get(outputPath, outputDirName)
//        logger.info("Cloning ${gitRepository.name} into $outputDirectory")
//
//        val git: Git = Git.cloneRepository()
//            .setURI(gitRepository.uri)
//            .setDirectory(outputDirectory.toFile())
//            .call()
//        git.close()
//
//        responseChannel(GitCloneDoneEvent(taskID, repoDbId, outputDirectory.toString()))
//        logger.info("Finished cloning ${gitRepository.name}")
//    }
//}

//
//import de.fraunhofer.iem.dataprovider.configuration.OpenCodeGitlabApiConfiguration
//import de.fraunhofer.iem.dataprovider.repository.service.RepositoryService
//import de.fraunhofer.iem.dataprovider.taskManager.events.Event
//import de.fraunhofer.iem.dataprovider.taskManager.tasks.tools.sarif.SarifTask
//import de.fraunhofer.iem.dataprovider.toolRun.service.ToolRunService
//import org.springframework.core.io.ClassPathResource
//import org.springframework.core.io.Resource
//import java.io.File
//import java.nio.file.Files
//import java.nio.file.Path
//import java.nio.file.Paths
//import java.util.*
//
//class OccmdTask(
//    repositoryDirectoryPath: String,
//    outputDirectoryPath: String,
//    openCodeGitlabApiConfiguration: OpenCodeGitlabApiConfiguration,
//    private val repositoryService: RepositoryService,
//    override val responseChannel: suspend (task: Event) -> Unit,
//    override val repoId: UUID,
//    override val groupID: UUID? = null,
//    override val toolRunService: ToolRunService
//) : SarifTask() {
//
//    private val resource: Resource = ClassPathResource("scripts/occmd.sh")
//
//    override val outputDirectory: String = Paths.get(outputDirectoryPath, "occmd", taskID.toString()).toString()
//    private val copiedRepositoryPath = Paths.get(outputDirectory, "copiedRepo")
//    override val getSarif = ::getOccmdSarifFromFilePath
//
//    private val repositoryID = getRepositoryId(repoId)
//    private val checkSecretPath =
//        Paths.get(outputDirectory, "app", "notes", "opencode.d", "code", "resources", "checks", "secrets")
//
//    override val flags: Array<String> = arrayOf(
//        resource.file.absolutePath,
//        "--key",
//        openCodeGitlabApiConfiguration.accessToken,
//        "--username",
//        "USERNAME", //TODO: Make sure is it required? if not delete otherwise dynamically take the username
//        "--proj-path",
//        copiedRepositoryPath.toString(),
//        "--out-dir",
//        Paths.get(outputDirectory, "app", "notes").toAbsolutePath().toString(),
//        "--console-out-file",
//        Paths.get(outputDirectory, "console.txt").toAbsolutePath().toString(),
//        "--proj-id",
//        repositoryID
//    )
//
//    override val resultFileOutputPath: Path =
//        Paths.get(checkSecretPath.toAbsolutePath().toString(), repositoryID, "DetectSecrets.baseline")
//
//    private fun getRepositoryId(repoId: UUID): String {
//        val repoPlatformId = repositoryService.findRepoByID(repoId)?.repoId
//            ?: throw NoSuchElementException("Repository with id $repoId was not found.")
//        return repoPlatformId.toString()
//    }
//
//
//    private val dbRawPath = Paths.get(outputDirectory, "app", "notes", "opencode.d", "code", "db", "raw")
//    private val blacklistPath = Paths.get(
//        outputDirectory,
//        "app",
//        "notes",
//        "opencode.d",
//        "code",
//        "resources",
//        "checks",
//        "checked_in_binaries",
//        "blacklist"
//    )
//
//    override fun cleanUp() {
//        File(copiedRepositoryPath.toUri()).deleteRecursively()
//    }
//
//    init {
//        val repoFile = File(repositoryDirectoryPath)
//        val copiedRepo = File(copiedRepositoryPath.toUri())
//        repoFile.copyRecursively(copiedRepo)
//
//        Files.createDirectories(dbRawPath)
//        Files.createDirectories(blacklistPath)
//        Files.createDirectories(checkSecretPath)
//    }
//}
